"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFileSystemStorage = void 0;
const storage_1 = require("@varasto/storage");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("./utils");
/**
 * Creates new file system storage with given options.
 */
const createFileSystemStorage = (options = {}) => {
    var _a, _b, _c, _d;
    const dir = (_a = options.dir) !== null && _a !== void 0 ? _a : './data';
    const encoding = (_b = options.encoding) !== null && _b !== void 0 ? _b : 'utf-8';
    const serialize = (_c = options.serialize) !== null && _c !== void 0 ? _c : JSON.stringify;
    const deserialize = (_d = options.deserialize) !== null && _d !== void 0 ? _d : JSON.parse;
    return {
        has: (namespace, key) => {
            let filename;
            try {
                filename = utils_1.buildFilename(dir, namespace, key);
            }
            catch (_a) {
                return Promise.resolve(false);
            }
            return Promise.resolve(fs_1.default.existsSync(filename));
        },
        keys: (namespace) => utils_1.globNamespace(dir, namespace).then((matches) => matches.map((match) => path_1.default.basename(match, '.json'))),
        values: (namespace) => utils_1.globNamespace(dir, namespace)
            .then((filenames) => Promise.all(filenames.map((filename) => utils_1.readItem(filename, encoding, deserialize))))
            .then((values) => values.filter((value) => value !== undefined)),
        entries: (namespace) => utils_1.globNamespace(dir, namespace)
            .then((filenames) => Promise.all(filenames.map((filename) => utils_1.readItem(filename, encoding, deserialize).then((value) => [
            path_1.default.basename(filename, '.json'),
            value,
        ]))))
            .then((entries) => entries.filter((entry) => entry[1] !== undefined)),
        get: (namespace, key) => {
            let filename;
            try {
                filename = utils_1.buildFilename(dir, namespace, key);
            }
            catch (err) {
                return Promise.reject(err);
            }
            return utils_1.readItem(filename, encoding, deserialize);
        },
        set: (namespace, key, value) => utils_1.createNamespace(dir, namespace).then(() => new Promise((resolve, reject) => {
            let filename;
            try {
                filename = utils_1.buildFilename(dir, namespace, key);
            }
            catch (err) {
                reject(err);
                return;
            }
            fs_1.default.writeFile(filename, serialize(value), encoding, (err) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        })),
        update: (namespace, key, value) => {
            let filename;
            try {
                filename = utils_1.buildFilename(dir, namespace, key);
            }
            catch (err) {
                return Promise.reject(err);
            }
            return utils_1.readItem(filename, encoding, deserialize).then((oldValue) => {
                if (oldValue !== undefined) {
                    const newValue = Object.assign(Object.assign({}, oldValue), value);
                    return new Promise((resolve, reject) => {
                        fs_1.default.writeFile(filename, serialize(newValue), encoding, (err) => {
                            if (err) {
                                reject(err);
                            }
                            else {
                                resolve(newValue);
                            }
                        });
                    });
                }
                return Promise.reject(new storage_1.ItemDoesNotExistError('Item does not exist'));
            });
        },
        delete: (namespace, key) => new Promise((resolve, reject) => {
            let filename;
            try {
                filename = utils_1.buildFilename(dir, namespace, key);
            }
            catch (err) {
                reject(err);
                return;
            }
            fs_1.default.exists(filename, (exists) => {
                if (!exists) {
                    resolve(false);
                    return;
                }
                fs_1.default.unlink(filename, (err) => {
                    if (err) {
                        if (err.code === 'ENOENT') {
                            resolve(false);
                        }
                        else {
                            reject(err);
                        }
                    }
                    else {
                        resolve(true);
                    }
                });
            });
        }),
    };
};
exports.createFileSystemStorage = createFileSystemStorage;
